from getpass import getpass
from RestClient.ErrorHandling.RestClientExceptions import ClientAuthException

import sys

class BasicAuth(object):
    def __init__(self, username=None, password=None):
        self._username = username
        self._password = password

        if sys.stdin.isatty() and not self._username:
            self._username = input('User:')

        if sys.stdin.isatty() and not self._password:
            self._password = getpass("Password:")

        if not (self._username and self._password):
            raise ClientAuthException("No valid user or password specified for BasicAuth")

    def configure_auth(self, curl_object):
        curl_object.setopt(curl_object.HTTPAUTH, curl_object.HTTPAUTH_BASIC)
        curl_object.setopt(curl_object.USERPWD, ("%s:%s") % (self.userpwd))

    @property
    def userpwd(self):
        return self._username, self._password



class ClientAuthException(Exception):
    def __init__(self, msg):
        self._msg = msg
        super(ClientAuthException, self).__init__(self, "ClientAuthException %s" % self._msg)

    def __repr__(self):
        return ('%s %r' % (self.__class__.__name__, self._msg))

    def __str__(self):
        return repr(self._msg)

class HTTPError(Exception):
    def __init__(self, url, code, msg, header, body):
        self.url = url
        self.code = code
        self.msg = msg
        self.header = header
        self.body = body
        super(HTTPError, self).__init__(self, "HTTPError %d" % self.code)

    def __repr__(self):
        return ('%s %r' % (self.__class__.__name__, self.code))

    def __str__(self):
        return ('HTTP Error %d: %s' % (self.code, self.msg))



from RestClient.RequestHandling.HTTPRequest import HTTPRequest
import pycurl
import certifi

class RestApi(object):
    def __init__(self, auth=None, proxy=None, additional_curl_options=None, use_shared_handle=False):
        self._curl = pycurl.Curl()

        if use_shared_handle:
            ###use shared Cookie, DNS and SSL Session ID caches when operating multi-threaded
            shared_curl = pycurl.CurlShare()
            shared_curl.setopt(pycurl.SH_SHARE, pycurl.LOCK_DATA_COOKIE)
            shared_curl.setopt(pycurl.SH_SHARE, pycurl.LOCK_DATA_DNS)
            shared_curl.setopt(pycurl.SH_SHARE, pycurl.LOCK_DATA_SSL_SESSION)
            self._curl.setopt(pycurl.SHARE, shared_curl)

        if additional_curl_options:
            self._additional_curl_options = additional_curl_options
        else:
            self._additional_curl_options = {}

        if auth:
            auth.configure_auth(self._curl)
        if proxy:
            proxy.configure_proxy(self._curl)
        # follow redirect link
        self._curl.setopt(pycurl.FOLLOWLOCATION, True)

    def __del__(self):
        self._curl.close()

    def get(self, url, api, params={}, data=None, request_headers={}):
        http_request = HTTPRequest(method='GET', url=url, api=api, params=params,
                                   data=data, request_headers=request_headers,
                                   additional_curl_options=self._additional_curl_options)
        return http_request(self._curl)

    def post(self, url, api, params={}, data="", request_headers={}):
        http_request = HTTPRequest(method='POST', url=url, api=api, params=params, data=data,
                                   request_headers=request_headers,
                                   additional_curl_options=self._additional_curl_options)
        return http_request(self._curl)

    def put(self, url, api, params={}, data="", request_headers={}):
        http_request = HTTPRequest(method='PUT', url=url, api=api, params=params, data=data,
                                   request_headers=request_headers,
                                   additional_curl_options=self._additional_curl_options)
        return http_request(self._curl)

    def delete(self, url, api, params={}, data=None, request_headers={}):
        http_request = HTTPRequest(method='DELETE', url=url, api=api, params=params, data=data,
                                   request_headers=request_headers,
                                   additional_curl_options=self._additional_curl_options)
        return http_request(self._curl)



from getpass import getpass
from RestClient.ErrorHandling.RestClientExceptions import ClientAuthException

import os, sys

class X509Auth(object):
    def __init__(self, ca_path=None, ssl_cert=None, ssl_key=None, ssl_verifypeer=True, ca_info=None):
        self._ca_path = ca_path
        self._ssl_cert = ssl_cert
        self._ssl_key = ssl_key
        self._ssl_verifypeer = ssl_verifypeer
        self._ca_info = ca_info
        if not (self._ssl_cert and self._ssl_key):
            self.__search_cert_key()

        if not self._ca_path:
            self.__search_ca_path()

        #Check if ssl_cert, ssl_key and ca_path do exist
        if not (os.path.isfile(self._ssl_key) and os.path.isfile(self._ssl_cert)):
            raise ClientAuthException("key or cert file does not exist: %s, %s" % (self._ssl_key, self._ssl_cert))

        if not (os.path.isdir(self._ca_path)):
            raise ClientAuthException("CA path does not exist: %s" % (self._ca_path))

    def __search_ca_path(self):
        """
        Get CA Path to check the validity of the server host certificate on the client side
        """
        if "X509_CERT_DIR" in os.environ:
            self._ca_path = os.environ['X509_CERT_DIR']

        elif os.path.exists('/etc/grid-security/certificates'):
            self._ca_path = '/etc/grid-security/certificates'

        else:
            raise ClientAuthException("Could not find a valid CA path")
    
    def __search_cert_key(self):
        """
        Get the user credentials if they exist, otherwise throw an exception.
        This code was modified from DBSAPI/dbsHttpService.py and WMCore/Services/Requests.py
        """
        # Now we're trying to guess what the right cert/key combo is...
        # First preference to HOST Certificate, This is how it set in Tier0
        if 'X509_HOST_CERT' in os.environ:
            self._ssl_cert = os.environ['X509_HOST_CERT']
            self._ssl_key = os.environ['X509_HOST_KEY']

        # Second preference to User Proxy, very common
        elif 'X509_USER_PROXY' in os.environ and os.path.exists(os.environ['X509_USER_PROXY']):
            self._ssl_cert = os.environ['X509_USER_PROXY']
            self._ssl_key = self._ssl_cert

        # Third preference to User Cert/Proxy combinition
        elif 'X509_USER_CERT' in os.environ and 'X509_USER_KEY' in os.environ:
            self._ssl_cert = os.environ['X509_USER_CERT']
            self._ssl_key = os.environ['X509_USER_KEY']

        # TODO: only in linux, unix case, add other os case
        # look for proxy at default location /tmp/x509up_u$uid
        elif os.path.exists('/tmp/x509up_u%s' % str(os.getuid())):
            self._ssl_cert = '/tmp/x509up_u%s' % str(os.getuid())
            self._ssl_key = self._ssl_cert

        elif sys.stdin.isatty():
            home_dir = os.environ['HOME']
            user_cert = os.path.join(home_dir, '.globus/usercert.pem')
            user_key = os.path.join(home_dir, '.globus/userkey.pem')

            if os.path.exists(user_cert):
                self._ssl_cert = user_cert
                if os.path.exists(user_key):
                    self._ssl_key = user_key
                    #store password for convenience
                    self._ssl_key_pass = getpass("Password for %s: " % self._ssl_key)
                else:
                    self._ssl_key = self._ssl_cert
            else:
                raise ClientAuthException("No valid X509 cert-key-pair found.")    

        else:
            raise ClientAuthException("No valid X509 cert-key-pair found.")

    def configure_auth(self, curl_object):
        curl_object.setopt(curl_object.SSL_VERIFYPEER, self._ssl_verifypeer)
        #not sure if CAPATH shoud be set. YG 2021-Oct-11
        curl_object.setopt(curl_object.CAPATH, self._ca_path)
        curl_object.setopt(curl_object.SSLCERT, self._ssl_cert)
        curl_object.setopt(curl_object.SSLKEY, self._ssl_key)
        if self._ca_info:
            pass
            # comment out as suggested. YG 2021-Oct-11
            #curl_object.setopt(curl_object.CAINFO, self._ca_info)

        if self.ssl_key_pass:
            curl_object.setopt(curl_object.SSLKEYPASSWD, self.ssl_key_pass)

    @property
    def ssl_key_pass(self):
        return getattr(self, '_ssl_key_pass', None)



from RestClient.RequestHandling.HTTPResponse import HTTPResponse
from RestClient.ErrorHandling.RestClientExceptions import HTTPError

import certifi
import pycurl
import urllib.request, urllib.parse, urllib.error

from io import BytesIO


class HTTPRequest(object):
    supported_methods = {'GET'    : {pycurl.HTTPGET : True},
                         'POST'   : {pycurl.POST : True},
                         'PUT'    : {pycurl.UPLOAD : True},#pycurl.PUT has been deprecated
                         'DELETE' : {pycurl.CUSTOMREQUEST : 'DELETE'}}

    def __init__(self, method, url, api, params, data, request_headers={}, additional_curl_options={}):
        method = method.upper()
        self._curl_options = dict(additional_curl_options) ### copy dict since mutables are shared between instances
        request_headers = dict(request_headers) ### copy dict since mutables are shared between instances

        try:
            self._curl_options.update(self.supported_methods[method])
        except KeyError as ke:
            raise NotImplementedError("HTTP method %s has not been implemented yet." % (ke))

        if not params:
            self._curl_options[pycurl.URL] = ("%s/%s") % (url, api)
        else:
            self._curl_options[pycurl.URL] = ("%s/%s?%s") % (url, api, urllib.parse.urlencode(params))

        if method == 'POST':
            self._curl_options[pycurl.POSTFIELDS] = data
            if not data: ###for zero-byte post this is a mandatory option
                self._curl_options[pycurl.POSTFIELDSIZE] = 0
            ### pycurl will automatically set content-length header using strlen()

        elif method == 'PUT':
            data_fp = BytesIO(data.encode())
            content_length = len(data)
            self._curl_options[pycurl.READFUNCTION] = data_fp.read
            self._curl_options[pycurl.INFILESIZE] = content_length
            ### set content-length header to ensure performant cherrypy reads
            request_headers['Content-Length'] = str(content_length)

        self._curl_options[pycurl.HTTPHEADER] = ["%s: %s" % (key, value) for key, value in request_headers.items()]

    def __call__(self, curl_object):
        for key, value in self._curl_options.items():
            curl_object.setopt(key, value)

        http_response  = HTTPResponse()
        curl_object.setopt(pycurl.HEADERFUNCTION, http_response.pycurl_header_function)
        curl_object.setopt(pycurl.WRITEFUNCTION, http_response.pycurl_write_function)

        curl_object.perform()

        http_code = curl_object.getinfo(pycurl.HTTP_CODE)

        if http_code < 200 or http_code >=300:
            effective_url = curl_object.getinfo(pycurl.EFFECTIVE_URL)
            raise HTTPError(effective_url, http_code, http_response.msg, http_response.raw_header, http_response.body)

        return http_response



from urllib.parse import urlparse


class Socks5Proxy(object):
    """Socks5 Proxy Plugin for pycurl
    proxy_url format socks5://username:passwd@hostname:port"""
    def __init__(self, proxy_url):
        parsed_url = urlparse(proxy_url)
        self._proxy_hostname = parsed_url.hostname
        self._proxy_port = parsed_url.port
        self._proxy_user = parsed_url.username
        self._proxy_passwd = parsed_url.password

    def configure_proxy(self, curl_object):
        """configure pycurl proxy settings"""
        curl_object.setopt(curl_object.PROXY, self._proxy_hostname)
        curl_object.setopt(curl_object.PROXYPORT, self._proxy_port)
        curl_object.setopt(curl_object.PROXYTYPE, curl_object.PROXYTYPE_SOCKS5)
        if self._proxy_user and self._proxy_passwd:
            curl_object.setopt(curl_object.PROXYUSERPWD, '%s:%s' % (self._proxy_user, self._proxy_port)) 



from __future__ import print_function
import sys, os, os.path, re, shutil, string
from distutils.core import setup, Command
from distutils.command.build import build
from distutils.command.install import install
from distutils.spawn import spawn
from glob import glob

systems = \
{'pycurl-client':
  {
    'srcdir': 'src/python',
    'pythonmods': ['RestClient.__init__',
                   'RestClient.RestApi'],
    'pythonpkg': ['RestClient.AuthHandling',
                  'RestClient.ErrorHandling',
                  'RestClient.ProxyPlugins',
                  'RestClient.RequestHandling']
  }
}
def get_relative_path():
      return os.path.dirname(os.path.abspath(os.path.join(os.getcwd(), sys.argv[0])))

def process_dependencies(system):
  srcdir = system['srcdir']
  binaries = set(system.get('bin', set()))
  examples = set(system.get('examples', set()))
  configs = set(system.get('conf', set()))
  data = set(system.get('data', set()))
  statics = set(system.get('statics', set()))
  pythonmods = set(system.get('pythonmods', set()))
  pythonpkg = set(system.get('pythonpkg', set()))

  dependencies = system.get('dependencies', [])

  for dependency in dependencies:
    dependant_system = systems[dependency]
    if dependant_system['srcdir'] != srcdir:
      print("Dependencies have to be in the same root directory")
      sys.exit(1)

    dependants = process_dependencies(dependant_system)
    binaries.update(dependants.get('bin', set()))
    examples.update(dependants.get('examples', set()))
    configs.update(dependants.get('conf', set()))
    data.update(dependants.get('data', set()))
    statics.update(dependants.get('statics', set()))
    pythonmods.update(dependants.get('pythonmods', set()))
    pythonpkg.update(dependants.get('pythonpkg', set()))

  return {'srcdir': srcdir,
          'bin': list(binaries),
          'examples': list(examples),
          'conf': list(configs),
          'data': list(data),
          'statics': list(statics),
          'pythonmods': list(pythonmods),
          'pythonpkg': list(pythonpkg)}

def define_the_build(self, dist, system_name, run_make = True, patch_x = ''):
  # Expand various sources.
  docroot = "doc/build/html"
  system = process_dependencies(systems[system_name])
  exsrc = sum((glob("%s" % x) for x in system.get('examples', [])), [])
  configs = sum((glob("%s" % x) for x in system.get('conf', [])), [])
  data = sum((glob("%s" % x) for x in system.get('data', [])), [])
  statics = sum((glob("%s" % x) for x in system.get('statics', [])), [])
  binsrc = sum((glob("%s" % x) for x in system.get('bin', [])), [])

  # Specify what to install.
  py_version = sys.version.split()[0]
  pylibdir = '%slib/python%s/site-packages' % (patch_x, py_version[0:3])
  dist.py_modules = system.get('pythonmods', [])
  dist.packages = system.get('pythonpkg', [])
  dist.package_dir = { '': system.get('srcdir', []) }
  dist.data_files = [('examples', exsrc), ('%sbin' % patch_x, binsrc),
                     ('conf', configs), ('data', data), ('statics', statics)]

  for directory in set(os.path.dirname(path.replace('Client/utils/', '', 1)) for path in exsrc):
      files = [x for x in exsrc if x.startswith('Client/utils/%s/' % directory)]
      dist.data_files.append(('examples/%s' % (directory), files))

  if os.path.exists(docroot):
    for dirpath, dirs, files in os.walk(docroot):
      dist.data_files.append(("%sdoc%s" % (patch_x, dirpath[len(docroot):]),
                              ["%s/%s" % (dirpath, fname) for fname in files
                               if fname != '.buildinfo']))

class BuildCommand(Command):
  """Build python modules for a specific system."""
  description = \
    "Build python modules for the specified system. Possible\n" + \
    "\t\t   systems are 'dbs-web', 'dbs-client', 'pycurl-client' or 'dbs-migration'.\n" + \
    "Use with --force to\n" + \
    "\t\t   ensure a clean build of only the requested parts.\n"
  user_options = build.user_options
  user_options.append(('system=', 's', 'build the specified system'))

  def initialize_options(self):
    self.system = None

  def finalize_options(self):
    # Check options.
    if self.system == None:
      print("System not specified, please use '-s dbs-web', '-s dbs-client', '-s pycurl-client' or '-s dbs-migration'")
      sys.exit(1)
    elif self.system not in systems:
      print("System %s unrecognised, please use '-s dbs-web', '-s dbs-client', '-s pycurl-client' or \
'-s dbs-migration'" % self.system)
      sys.exit(1)

    # Expand various sources and maybe do the c++ build.
    define_the_build(self, self.distribution, self.system, True, '')

    # Force rebuild.
    shutil.rmtree("%s/build" % get_relative_path(), True)
    shutil.rmtree("doc/build", True)

  def generate_docs(self):
    if self.system=="dbs-web":
      os.environ["PYTHONPATH"] = "%s/WMCore/build/lib/:%s" % (os.path.dirname(os.getcwd()), os.environ["PYTHONPATH"])
    os.environ["PYTHONPATH"] = "%s/build/lib:%s" % (os.getcwd(), os.environ["PYTHONPATH"])
    #spawn(['make', '-C', 'doc', 'html', 'PROJECT=%s' % self.system.lower()])
    spawn(['make', '-C', 'doc', 'html', 'PROJECT=dbs'])

  def run(self):
    command = 'build'
    if self.distribution.have_run.get(command): return
    cmd = self.distribution.get_command_obj(command)
    cmd.force = self.force
    cmd.ensure_finalized()
    cmd.run()
    # Stop calling generate_docs temporary
    # self.generate_docs()
    self.distribution.have_run[command] = 1

class InstallCommand(install):
  """Install a specific system."""
  description = \
    "Install a specific system, either 'dbs-web', 'dbs-client', 'pycurl-client' or 'dbs-migration'. You can\n" + \
    "\t\t   patch an existing installation instead of normal full installation\n" + \
    "\t\t   using the '-p' option.\n"
  user_options = install.user_options
  user_options.append(('system=', 's', 'install the specified system'))
  user_options.append(('patch', None, 'patch an existing installation'))

  def initialize_options(self):
    install.initialize_options(self)
    self.system = None
    self.patch = None

  def finalize_options(self):
    # Check options.
    if self.system == None:
      print("System not specified, please use '-s dbs-web', 'dbs-client', 'pycurl-client', 'dbs-migration',\
'LifeCycleTests' or 'LifeCycleAnalysis'")
      sys.exit(1)
    elif self.system not in systems:
      print("System %s unrecognised, please use '-s dbs-web', 'dbs-client', 'pycurl-client', 'dbs-migration',\
'LifeCycleTests' or 'LifeCycleAnalysis'" % self.system)
      sys.exit(1)
    if self.patch and not os.path.isdir("%s/xbin" % self.prefix):
      print("Patch destination %s does not look like a valid location." % self.prefix)
      sys.exit(1)

    # Expand various sources, but don't build anything from c++ now.
    define_the_build(self, self.distribution, self.system,
		     False, (self.patch and 'x') or '')

    # Whack the metadata name.
    self.distribution.metadata.name = self.system
    assert self.distribution.get_name() == self.system

    # Pass to base class.
    install.finalize_options(self)

    # Mangle paths if we are patching. Most of the mangling occurs
    # already in define_the_build(), but we need to fix up others.
    if self.patch:
      self.install_lib = re.sub(r'(.*)/lib/python(.*)', r'\1/xlib/python\2', self.install_lib)
      self.install_scripts = re.sub(r'(.*)/bin$', r'\1/xbin', self.install_scripts)

  def run(self):
    for cmd_name in self.get_sub_commands():
      cmd = self.distribution.get_command_obj(cmd_name)
      cmd.distribution = self.distribution
      if cmd_name == 'install_data':
        cmd.install_dir = self.prefix
      else:
        cmd.install_dir = self.install_lib
      cmd.ensure_finalized()
      self.run_command(cmd_name)
      self.distribution.have_run[cmd_name] = 1

setup(name = 'dbs',
      version = '3.4.4',
      maintainer_email = 'hn-cms-dmDevelopment@cern.ch',
      cmdclass = {'build_system': BuildCommand,
                  'install_system': InstallCommand})


#!/usr/bin/env python

import os
import sys
from setuptools import setup

# version of pycurl-client, should match current DBS release tag
package_version = "3.17.8"

# Requirements file for pip dependencies
requirements = "requirements.txt"


def parse_requirements(requirements_file):
    """
      Create a list for the 'install_requires' component of the setup function
      by parsing a requirements file
    """

    if os.path.exists(requirements_file):
        # return a list that contains each line of the requirements file
        return open(requirements_file, 'r').read().splitlines()
    else:
        print("ERROR: requirements file " + requirements_file + " not found.")
        sys.exit(1)

setup(name="dbs3-pycurl",
      version=package_version,
      maintainer="CMS DWMWM Group",
      maintainer_email="hn-cms-dmDevelopment@cern.ch",
      packages=['RestClient',
                'RestClient.AuthHandling',
                'RestClient.ErrorHandling',
                'RestClient.ProxyPlugins',
                'RestClient.RequestHandling'],
      package_dir={'': 'src/python/'},
      install_requires=parse_requirements(requirements),
      url="https://github.com/dmwm/PycurlClient/tree/3.17.8",
      license="Apache License, Version 2.0",
      )



from io import BytesIO


class HTTPResponse(object):
    def __init__(self):
        self._response_header = BytesIO()
        self._response_body = BytesIO()

    def __parse_header(self):
        self._header_dict = {}
        for header in self._response_header.getvalue().decode().split('\r\n'):
            if header.startswith('HTTP'):
                #self._version, self._code, self._msg = header.split(' ', 2)
                initial_header = header.split(' ', 2)
                self._version, self._code = initial_header[0:2]
                self._msg = initial_header[2] if len(initial_header) > 2 else '' 
            elif header != "":
                self._header_dict.update(dict([header.split(':', 1)]))

    @property
    def body(self):
        """Returns raw body received from server"""
        return self._response_body.getvalue().decode()

    @property
    def fp_body(self):
        """Returns body file pointer equivalent from BytesIO.
        For example to read data directly using json.load(fp)"""
        self._response_body.seek(0)
        return self._response_body

    @property
    def raw_header(self):
        """Returns raw response header received from server"""
        return self._response_header.getvalue().decode()
    
    @property
    def header(self):
        """Returns header dictionary created by parsing raw response header from server"""
        if not hasattr(self, '_header_dict'):
            self.__parse_header()
        return self._header_dict

    @property
    def pycurl_write_function(self):
        """Returns body write function from BytesIO.
        To be used as pycurl.WRITEFUNCTION"""
        return self._response_body.write

    @property
    def pycurl_header_function(self):
        """Returns header write function from BytesIO.
        To be used as pycurl.HEADERFUNCTION"""
        return self._response_header.write

    @property
    def code(self):
        """Returns http code from server response header"""
        if not hasattr(self, '_code'):
            self.__parse_header()
        return self._code

    @property
    def msg(self):
        """Returns message from server response header"""
        if not hasattr(self, '_msg'):
            self.__parse_header()
        return self._msg

    @property
    def version(self):
        """Returns used HTTP version from server response header"""
        if not hasattr(self, '_version'):
            self.__parse_header()
        return self._version
